@@ -21,12 +21,12 @@ import six
 
 from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import models as db_models
+from mistral.engine import action_queue
 from mistral.engine import dispatcher
 from mistral.engine import utils as engine_utils
 from mistral import exceptions as exc
+from mistral import expressions as expr
 from mistral.lang import parser as spec_parser
-from mistral.rpc import clients as rpc
-from mistral.services import scheduler
 from mistral.services import triggers
 from mistral.services import workflows as wf_service
 from mistral import utils
@@ -42,10 +42,6 @@ from mistral_lib import actions as ml_actions
 
 LOG = logging.getLogger(__name__)
 
-_SEND_RESULT_TO_PARENT_WORKFLOW_PATH = (
-    'mistral.engine.workflows._send_result_to_parent_workflow'
-)
-
 
 @six.add_metaclass(abc.ABCMeta)
 class Workflow(object):
@@ -118,12 +114,11 @@ class Workflow(object):
         assert self.wf_ex
 
         if state == states.SUCCESS:
-            return self._succeed_workflow(self._get_final_context(), msg)
+            self._succeed_workflow(self._get_final_context(), msg)
         elif state == states.ERROR:
-            return self._fail_workflow(self._get_final_context(), msg)
+            self._fail_workflow(self._get_final_context(), msg)
         elif state == states.CANCELLED:
-
-            return self._cancel_workflow(msg)
+            self._cancel_workflow(msg)
 
     def pause(self, msg=None):
         """Pause workflow.
@@ -244,10 +239,12 @@ class Workflow(object):
         return db_api.acquire_lock(db_models.WorkflowExecution, self.wf_ex.id)
 
     def _get_final_context(self):
+        final_ctx = {}
+
         wf_ctrl = wf_base.get_controller(self.wf_ex)
-        final_context = {}
+
         try:
-            final_context = wf_ctrl.evaluate_workflow_final_context()
+            final_ctx = wf_ctrl.evaluate_workflow_final_context()
         except Exception as e:
             LOG.warning(
                 'Failed to get final context for workflow execution. '
@@ -257,7 +254,7 @@ class Workflow(object):
                 str(e)
             )
 
-        return final_context
+        return final_ctx
 
     def _create_execution(self, wf_def, input_dict, desc, params):
         self.wf_ex = db_api.create_workflow_execution({
@@ -270,6 +267,7 @@ class Workflow(object):
             'state': states.IDLE,
             'output': {},
             'task_execution_id': params.get('task_execution_id'),
+            'root_execution_id': params.get('root_execution_id'),
             'runtime_context': {
                 'index': params.get('index', 0)
             },
@@ -277,16 +275,12 @@ class Workflow(object):
 
         self.wf_ex.input = input_dict or {}
 
-        env = _get_environment(params)
-
-        if env:
-            params['env'] = env
+        params['env'] = _get_environment(params)
 
         self.wf_ex.params = params
 
         data_flow.add_openstack_data_to_context(self.wf_ex)
         data_flow.add_execution_to_context(self.wf_ex)
-        data_flow.add_environment_to_context(self.wf_ex)
         data_flow.add_workflow_variables_to_context(self.wf_ex, self.wf_spec)
 
         spec_parser.cache_workflow_spec_by_execution_id(
@@ -301,7 +295,17 @@ class Workflow(object):
         cur_state = self.wf_ex.state
 
         if states.is_valid_transition(cur_state, state):
-            self.wf_ex.state = state
+            wf_ex = db_api.update_workflow_execution_state(
+                id=self.wf_ex.id,
+                cur_state=cur_state,
+                state=state
+            )
+
+            if wf_ex is None:
+                # Do nothing because the state was updated previously.
+                return
+
+            self.wf_ex = wf_ex
             self.wf_ex.state_info = state_info
 
             wf_trace.info(
@@ -378,6 +382,7 @@ class Workflow(object):
         else:
             msg = _build_fail_info_message(wf_ctrl, self.wf_ex)
             final_context = wf_ctrl.evaluate_workflow_final_context()
+
             self._fail_workflow(final_context, msg)
 
         return 0
@@ -393,13 +398,14 @@ class Workflow(object):
         self.set_state(states.SUCCESS, msg)
 
         if self.wf_ex.task_execution_id:
-            self._schedule_send_result_to_parent_workflow()
+            self._send_result_to_parent_workflow()
 
     def _fail_workflow(self, final_context, msg):
         if states.is_paused_or_completed(self.wf_ex.state):
             return
 
         output_on_error = {}
+
         try:
             output_on_error = data_flow.evaluate_workflow_output(
                 self.wf_ex,
@@ -426,7 +432,7 @@ class Workflow(object):
         self.wf_ex.output = merge_dicts({'result': msg}, output_on_error)
 
         if self.wf_ex.task_execution_id:
-            self._schedule_send_result_to_parent_workflow()
+            self._send_result_to_parent_workflow()
 
     def _cancel_workflow(self, msg):
         if states.is_completed(self.wf_ex.state):
@@ -444,24 +450,47 @@ class Workflow(object):
         self.wf_ex.output = {'result': msg}
 
         if self.wf_ex.task_execution_id:
-            self._schedule_send_result_to_parent_workflow()
+            self._send_result_to_parent_workflow()
 
-    def _schedule_send_result_to_parent_workflow(self):
-        scheduler.schedule_call(
-            None,
-            _SEND_RESULT_TO_PARENT_WORKFLOW_PATH,
-            0,
-            wf_ex_id=self.wf_ex.id
+    def _send_result_to_parent_workflow(self):
+        if self.wf_ex.state == states.SUCCESS:
+            result = ml_actions.Result(data=self.wf_ex.output)
+        elif self.wf_ex.state == states.ERROR:
+            err_msg = (
+                self.wf_ex.state_info or
+                'Failed subworkflow [execution_id=%s]' % self.wf_ex.id
+            )
+
+            result = ml_actions.Result(error=err_msg)
+        elif self.wf_ex.state == states.CANCELLED:
+            err_msg = (
+                self.wf_ex.state_info or
+                'Cancelled subworkflow [execution_id=%s]' % self.wf_ex.id
+            )
+
+            result = ml_actions.Result(error=err_msg, cancel=True)
+        else:
+            raise RuntimeError(
+                "Method _send_result_to_parent_workflow() must never be called"
+                " if a workflow is not in SUCCESS, ERROR or CANCELLED state."
+            )
+
+        action_queue.schedule_on_action_complete(
+            self.wf_ex.id,
+            result,
+            wf_action=True
         )
 
 
 def _get_environment(params):
     env = params.get('env', {})
 
-    if isinstance(env, dict):
-        return env
+    if not env:
+        return {}
 
-    if isinstance(env, six.string_types):
+    if isinstance(env, dict):
+        env_dict = env
+    elif isinstance(env, six.string_types):
         env_db = db_api.load_environment(env)
 
         if not env_db:
@@ -469,47 +498,18 @@ def _get_environment(params):
                 'Environment is not found: %s' % env
             )
 
-        return env_db.variables
-
-    raise exc.InputException(
-        'Unexpected value type for environment [env=%s, type=%s]'
-        % (env, type(env))
-    )
-
-
-def _send_result_to_parent_workflow(wf_ex_id):
-    with db_api.transaction():
-        wf_ex = db_api.get_workflow_execution(wf_ex_id)
-
-        wf_output = wf_ex.output
-
-    if wf_ex.state == states.SUCCESS:
-        result = ml_actions.Result(data=wf_output)
-    elif wf_ex.state == states.ERROR:
-        err_msg = (
-            wf_ex.state_info or
-            'Failed subworkflow [execution_id=%s]' % wf_ex.id
-        )
-
-        result = ml_actions.Result(error=err_msg)
-    elif wf_ex.state == states.CANCELLED:
-        err_msg = (
-            wf_ex.state_info or
-            'Cancelled subworkflow [execution_id=%s]' % wf_ex.id
-        )
-
-        result = ml_actions.Result(error=err_msg, cancel=True)
+        env_dict = env_db.variables
     else:
-        raise RuntimeError(
-            "Method _send_result_to_parent_workflow() must never be called"
-            " if a workflow is not in SUCCESS, ERROR or CANCELLED state."
+        raise exc.InputException(
+            'Unexpected value type for environment [env=%s, type=%s]'
+            % (env, type(env))
         )
 
-    rpc.get_engine_client().on_action_complete(
-        wf_ex.id,
-        result,
-        wf_action=True
-    )
+    if ('evaluate_env' in params and
+            not params['evaluate_env']):
+        return env_dict
+    else:
+        return expr.evaluate_recursively(env_dict, {'__env': env_dict})
 
 
 def _build_fail_info_message(wf_ctrl, wf_ex):
