@@ -40,7 +40,7 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wf_service.create_workflows(workflow)
 
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         self.engine.stop_workflow(
             wf_ex.id,
@@ -71,6 +71,33 @@ class WorkflowCancelTest(base.EngineTestCase):
         self.assertEqual(1, len(task_execs))
         self.assertEqual(states.SUCCESS, task_1_ex.state)
 
+    def test_cancel_workflow_if_definition_deleted(self):
+        workflow = """
+        version: '2.0'
+
+        wf:
+          type: direct
+          tasks:
+            task1:
+              action: std.echo output="foo"
+              wait-before: 5
+        """
+
+        wf = wf_service.create_workflows(workflow)[0]
+
+        wf_ex = self.engine.start_workflow('wf', '', {})
+
+        with db_api.transaction():
+            db_api.delete_workflow_definition(wf.id)
+
+        self.engine.stop_workflow(
+            wf_ex.id,
+            states.CANCELLED,
+            "Cancelled by user."
+        )
+
+        self.await_workflow_cancelled(wf_ex.id)
+
     def test_cancel_paused_workflow(self):
         workflow = """
         version: '2.0'
@@ -90,7 +117,7 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wf_service.create_workflows(workflow)
 
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         self.engine.pause_workflow(wf_ex.id)
 
@@ -141,7 +168,7 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wf_service.create_workflows(workflow)
 
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         self.await_workflow_success(wf_ex.id)
 
@@ -191,7 +218,7 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wb_service.create_workbook_v2(workbook)
 
-        wf_ex = self.engine.start_workflow('wb.wf', {})
+        wf_ex = self.engine.start_workflow('wb.wf', '', {})
 
         self.engine.stop_workflow(
             wf_ex.id,
@@ -257,7 +284,7 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wb_service.create_workbook_v2(workbook)
 
-        self.engine.start_workflow('wb.wf', {})
+        self.engine.start_workflow('wb.wf', '', {})
 
         with db_api.transaction():
             wf_execs = db_api.get_workflow_executions()
@@ -324,7 +351,7 @@ class WorkflowCancelTest(base.EngineTestCase):
         """
         wb_service.create_workbook_v2(workbook)
 
-        wf_ex = self.engine.start_workflow('wb.wf', {})
+        wf_ex = self.engine.start_workflow('wb.wf', '', {})
 
         self.engine.stop_workflow(
             wf_ex.id,
@@ -394,7 +421,7 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wb_service.create_workbook_v2(workbook)
 
-        self.engine.start_workflow('wb.wf', {})
+        self.engine.start_workflow('wb.wf', '', {})
 
         with db_api.transaction():
             wf_execs = db_api.get_workflow_executions()
@@ -473,7 +500,7 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wb_service.create_workbook_v2(workbook)
 
-        self.engine.start_workflow('wb.wf', {})
+        self.engine.start_workflow('wb.wf', '', {})
 
         with db_api.transaction():
             wf_execs = db_api.get_workflow_executions()
@@ -558,7 +585,7 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wb_service.create_workbook_v2(workbook)
 
-        self.engine.start_workflow('wb.wf', {})
+        self.engine.start_workflow('wb.wf', '', {})
 
         with db_api.transaction():
             wf_execs = db_api.get_workflow_executions()
